/***************************************************************************************
 * @file     bootloader.h
 * @version  V1.0
 * @date     10 September 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2018 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
#ifndef __BOOTLOADER_H
#define __BOOTLOADER_H

/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include "board.h"

/*****************************************************************************/ 
/* Global pre-processor symbols/macros ('#define')                           */ 
/*****************************************************************************/

/* PC MODE */
/* Prints more verbose programming information, ideal for use with a terminal
   program such as PuTTY. Undef for serial MCU<->MCU mode to remove excess prints */
#define PC_MODE

/* REB1_M95M01 */
/* When BRD_REB1 defined, use programming algorithm for SPI EEPROM, specifically M95M01
   128kB EEPROM used on the REB1 evaluation board. Undef for use of F-RAM 
	 instead of SPI EEPROM */
#ifdef BRD_REB1
#define REB1_M95M01
#endif

/* Bootloader timeout */
/* Defines the amount of time (in milliseconds) to wait for the heralding 
   character to indicate intent to upload a new user application */
#ifdef PC_MODE
#define BOOTLOADER_MSEC_TIMEOUT	5000
#else
#define BOOTLOADER_MSEC_TIMEOUT	100
#endif

/* Bootloader heralding character */
/* This is the character that the bootloader will look for on the UART
   to indicate intent to upload a new user application */
#define BOOTLOADER_HERALDING_CHAR ' ' // ASCII 'space' character
/* and this is the response sent when in MCU<->MCU mode */
#define BOOTLOADER_HERALDING_RESP_CHAR 'C' // ASCII 'C' character

/* Important bootloader addresses and offsets, vector table information */
#define BOOTLOADER_START_ADDR    (0x0)
#define BOOTLOADER_END_ADDR      (0x2000)
#define BOOTLOADER_CRC_ADDR      (0x1FFE)
#define APPLICATION_A_START_ADDR (0x2000)
#define APPLICATION_A_END_ADDR   (0x11000)
#define APPLICATION_A_CRC_ADDR   (0x10FFE)
#define APPLICATION_B_START_ADDR (0x11000)
#define APPLICATION_B_END_ADDR   (0x20000)
#define APPLICATION_B_CRC_ADDR   (0x1FFFE)
#define APPLICATION_IMG_SZ       (0xF000)
#define VECTOR_TABLE_OFFSET		   (0x0)
#define VECTOR_TABLE_LEN         (0xC0)
#define RESET_VECTOR_OFFSET      (0x4)

/*****************************************************************************/
/* Global type definitions ('typedef')                                       */ 
/*****************************************************************************/

typedef enum{
  en_memcheck_no_app_valid = 0,
  en_memcheck_app_a_valid = 1,
  en_memcheck_app_b_valid = 2
} en_memcheck_t;

typedef enum{
  en_runapp_a,
  en_runapp_b
} en_runapp_t;

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

extern en_memcheck_t Bootloader_CheckAppIsValid(void);
extern void Bootloader_RunApp(en_runapp_t);
extern void Bootloader_Task(void);
extern void Bootloader_CheckOwnCRC(void);

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif /* __BOOTLOADER_H */
