/***************************************************************************************
 * @file     va108xx_hal_spi.h
 * @version  V0.2
 * @date     22 February 2019
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2019 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
#ifndef __HAL_SPI_H
#define __HAL_SPI_H

/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include "va108xx_hal.h"
#include "event_handler_index.h"

/*****************************************************************************/ 
/* Global pre-processor symbols/macros ('#define')                           */ 
/*****************************************************************************/

#define HAL_SPI_VERSION     (0x00000203) // 0.2.3

#define NUM_SPI_BANKS       (3)

#define SPI_MASTER_MSK      (0xF) // SPI 0-3 support master mode
#define SPI_SLAVE_MSK       (0x7) // SPI 0-2 support slave mode

#define SPI0_BANK           (0)
#define SPI1_BANK           (1)
#define SPI2_BANK           (2)

#define SPI_PERIPHERAL_CLK  (SystemCoreClock)

#define SPI_MIN_WORDLEN     (4)
#define SPI_MAX_WORDLEN     (16)

#define VOR_SPI0            (VOR_SPIA)
#define VOR_SPI1            (VOR_SPIB)
#define VOR_SPI2            (VOR_SPIC)

// missing from header
/* ----------------------------------  SPI_DATA  --------------------------- */
#define SPI_DATA_VALUE_Pos   			 					0                                                      /*!< SPIB DATA: VALUE Position 							*/
#define SPI_DATA_VALUE_Msk   			 					(0x000000ffffUL << SPI_DATA_VALUE_Pos)   						 /*!< SPIB DATA: VALUE Mask   								*/
#define SPI_DATA_BMSKIPDATA_Pos   	 					30                                                     /*!< SPIB DATA: BMSKIPDATA Position 					*/
#define SPI_DATA_BMSKIPDATA_Msk   	 					(0x01UL << SPI_DATA_BMSKIPDATA_Pos)         					 /*!< SPIB DATA: BMSKIPDATA Mask   						*/
#define SPI_DATA_BMSTART_BMSTOP_Pos 					31                                                     /*!< SPIB DATA: BMSTART/BMSTOP Position 			*/
#define SPI_DATA_BMSTART_BMSTOP_Msk 					(0x01UL << SPI_DATA_BMSTART_BMSTOP_Pos)     					 /*!< SPIB DATA: BMSTART/BMSTOP Mask   				*/

/*****************************************************************************/
/* Global type definitions ('typedef')                                       */ 
/*****************************************************************************/

/** SPI clokcing mode */
typedef enum
{
  en_spi_clkmode_0 = 0,
  en_spi_clkmode_1 = 1,
  en_spi_clkmode_2 = 2,
  en_spi_clkmode_3 = 3
} en_spi_clkmode_t;

/** SPI master or slave configuration */
typedef enum 
{
  en_spi_ms_master = 0,
  en_spi_ms_slave = 1
} en_spi_ms_t;

/** The SPI initialization struct */
typedef struct
{
  en_spi_ms_t ms;
  en_spi_clkmode_t mode;
  uint16_t clkDiv;       // clock divider must be even
  uint8_t wordLen;
  //uint8_t slaveSelect; // moved into the transfer struct
  bool loopback;
  bool blockmode;
  //bool bmstart; // removed, this ctrl bit will be set to true
  bool bmstall;
  bool mdlycap;
} hal_spi_init_t;

/** The SPI transfer struct */
typedef struct 
{
  uint16_t *txbuf;   // transmit buffer address, or NULL for rx only (send pump)
  uint16_t *rxbuf;   // receive buffer address, or NULL for tx only
  uint16_t len;      // transfer length (must be nonzero)
  uint8_t  ss;       // slave select number
  bool     bmstop;   // Deassert CSn on last word of the xfer if blockmode (when master)
  eventHandlerIdx_t callbackIdx; // index in event handler list of function to call when complete
  volatile bool *complete; // address of bool to set true when xfer complete (NULL to ignore)
} spi_xfer_t;

/*****************************************************************************/
/* Global variable declarations ('extern', definition in C source)           */
/*****************************************************************************/

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

extern hal_status_t HAL_Spi_Init   (VOR_SPI_PERIPHERAL_Type* const spi, const hal_spi_init_t init);
extern hal_status_t HAL_Spi_DeInit (VOR_SPI_PERIPHERAL_Type* const spi);

extern hal_status_t HAL_Spi_InitMaster(VOR_SPI_PERIPHERAL_Type* const spi, uint16_t clkDiv, en_spi_clkmode_t mode, uint8_t wordLen);
extern hal_status_t HAL_Spi_InitSlave (VOR_SPI_PERIPHERAL_Type* const spi, uint16_t clkDiv, en_spi_clkmode_t mode, uint8_t wordLen);

extern hal_status_t HAL_Spi_Xfer   (VOR_SPI_PERIPHERAL_Type* const spi, spi_xfer_t* xfer);
extern hal_status_t HAL_Spi_XferB  (VOR_SPI_PERIPHERAL_Type* const spi, uint16_t* txBuf, uint16_t* rxBuf, uint16_t len, bool bmstop, uint8_t ss);

extern hal_status_t HAL_Spi_IsBusy (VOR_SPI_PERIPHERAL_Type* const spi, bool* busy);

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif /* __HAL_SPI_H */
